##################################################################################
#     Script: CIMS Aggregation Engine Perl Module
#
#     The information in this  document is subject to  change  without  notice
#     and should not be construed as a commitment by CIMS Lab, Inc.
#     CIMS Lab, Inc.  assumes no responsibility for any errors that
#     may appear in this document.
#
#     The software described in this  document is  furnished to the  purchaser
#     under  a license for use on a single  computer  system and can be copied
#     (with inclusion of CIMS Lab's copyright) only for use in such system,
#     except as  may  otherwise be provided  in writing  from CIMS Lab, Inc.
#
#     All rights reserved.  No part  of this document may be reproduced in any
#     form  or  by any  means,  except as may otherwise be provided in writing
#     from CIMS Lab, Inc.
#
#     Copyright (c) 1979 - 2005 by
#
#     CIMS Lab, Inc., Laurel Lab
#     14440 Cherry Lane Court   Telephone:  (301)-725-2500
#     Suite 201                 Fax      :  (301)-725-7196
#     Laurel, MD 20707
#     USA
#
# The CS_aggtool.pm Perl Module contains modules called by CIMS Process Job scripts.
#
#  CS_aggtool.pm Development History
#
#       Original Author   : George C. Bonifant, CIMS Lab, Inc., Laurel Lab
#       Creation Date     : 10-Feb-2005
#       Last Modification : 17-May-2005
#       Version Number    : V1.2
#
#  CS_aggtool.pm Modification Log
#
#       GCB=George C. Bonifant, CIMS Lab, Inc., Laurel Lab
#
#       GCB V1.1 10-Feb-2005
#
#               Created this module.
#
#	GCB V1.2 26-Apr-2006
#
#		Output metrics in six point decimal percision.
#
##################################################################################
#
package CIMS_aggtools;

use CIMS_lib::CS_libs;

$records_loaded = 0;
@my_ratecodes;
@my_aggids;
$my_aggheader = "";
$num_metrics = 0;
$num_aggids = 0;

$num_hashes = 9;

%stat_hash1;
%stat_hash2;
%stat_hash3;
%stat_hash4;
%stat_hash5;
%stat_hash6;
%stat_hash7;
%stat_hash8;
%stat_hash9;

%id_hash1;
%id_hash2;
%id_hash3;
%id_hash4;
%id_hash5;
%id_hash6;
%id_hash7;
%id_hash8;
%id_hash9;

%agg_start_times;
%agg_end_times;

BEGIN
{
    $log_separator = "*";
    $log_separator x= 80;
    $log_separator .= "\n";
}

################################################################################
################################################################################
#
sub cims_agg_clearengine() 
{
    my @hash_keys;
    my $hkey;
    my @id_keys;
    my $ikey;

    $count = 0;
    while ($count < $num_hashes) {
        ++$count;

        $this_hash = "stat_hash" . $count;
        @hash_keys = keys %$this_hash;
        foreach $hkey (@hash_keys) {
            delete $$this_hash{$hkey};
        }

        $this_hash = "id_hash" . $count;
        @id_keys = keys %$this_hash;
        foreach $ikey (@id_keys) {
            delete $$this_hash{$ikey};
        }
    }
}

################################################################################
################################################################################
#
sub cims_agg_initheader($) 
{
    ($my_aggheader) = @_;
}

################################################################################
################################################################################
#
sub cims_agg_initratecodes 
{
    @my_ratecodes = ();
    foreach $rcode (@_) {
        push(@my_ratecodes,$rcode);
    }
    $num_metrics = scalar(@my_ratecodes);
}

################################################################################
################################################################################
#
sub cims_agg_initids 
{
    @my_aggids = ();
    foreach $id (@_) {
        push(@my_aggids,$id);
    }
    $num_aggids = scalar(@my_aggids);
}


################################################################################
################################################################################
#
sub cims_agg_update 
{
    if ( scalar(@_) < 4 ) {
        printf "cims_agg_update: Usage cims_agg_update(id1[,idn],stat1[,statn],start,end)\n";
        return;
    }
    if ( $num_metrics == 0 || $num_aggids == 0 ) {
        printf "cims_agg_update: RateCode or ID array has not been initialized\n";
        return;
    }

    if ( ($num_metrics + $num_aggids + 2) != scalar(@_) ) {
        printf "cims_agg_update: Number of Arguments does not match initilization\n";
        return;
    }

##################################################################
# Diagnostic information
##################################################################
#
#    $count = 0;
#    foreach $rcode (@my_ratecodes) {
#        ++$count;
#        printf "    cims_agg_update: RATE_CODE %s = %s\n",$count,$rcode;
#    }
#
#    $count = 0;
#    foreach $aggid (@my_aggids) {
#        ++$count;
#        printf "    cims_agg_update: ID %s = %s\n",$count,$aggid;
#    }

    ############################################################################
    # Build Hash Key and load id array for this record.
    ############################################################################
    #
    $count = 0;
    $this_hkey = "";
    @id_list = ();
    while ( $count < $num_aggids ) {
        $this_id = shift(@_);
        push(@id_list,$this_id);
        $this_hkey = $this_hkey . $this_id;
        ++$count;
    }
##################################################################
# Diagnostic information
##################################################################
#
#    printf "    cims_agg_update: HKEY = %s\n",$this_hkey;

    ############################################################################
    # Load metric array for this record.
    ############################################################################
    #
    $count = 0;
    while ( $count < $num_metrics ) {
        $this_metric = shift(@_);
        push(@metric_list,$this_metric);
        ++$count;
    }

    ############################################################################
    # Load start and end times for this record.
    ############################################################################
    #
    $rec_start = shift(@_);
    $rec_end = shift(@_);

    ############################################################################
    # Load the id hashes
    #  Only do this if id values have not been loaded already
    ############################################################################
    #
    if ( length($id_hash1{$this_hkey}) == 0 ) { 
        $count = 0;
        $id_num = scalar(@id_list);
        while ( $count < $id_num ) {
            $this_id = shift(@id_list);
            ++$count;
            $id_hashx = "id_hash" . $count;
            $$id_hashx{$this_hkey} = $this_id;
##################################################################
# Diagnostic information
##################################################################
#
#            printf "            this id = %s\n",$$id_hashx{$this_hkey};
        }
    }

    ############################################################################
    # Update the metrics
    ############################################################################
    #
    $count = 0;
    while ($count < $num_metrics ) {
        $this_metric = shift(@metric_list);
##################################################################
# Diagnostic information
##################################################################
#
#        printf "            this_metric %s = %s\n",$count,$this_metric;
        ++$count;
        $stat_hashx = "stat_hash" . $count;
        $$stat_hashx{$this_hkey} = $$stat_hashx{$this_hkey} + $this_metric;
##################################################################
# Diagnostic information
##################################################################
#
#        printf "            stored_metric %s = %s\n",$count,$$stat_hashx{$this_hkey};
    }

    ############################################################################
    # Set start and end times hash tables.
    ############################################################################
    #
    if (( $agg_start_times{$this_hkey} == 0 ) || ( $agg_start_times{$this_hkey} > $rec_start )) {
        $agg_start_times{$this_hkey} = $rec_start;
    }

    if (( $agg_end_times{$this_hkey} == 0 ) || ( $agg_end_times{$this_hkey} < $rec_end )) {
        $agg_end_times{$this_hkey} = $rec_end;
    }
        
    ++$records_loaded;
}





################################################################################
################################################################################
#
sub cims_agg_genCSR(*) 
{
    my $lines_written = 0;
    $my_outhandle = shift;

    if ( $num_metrics == 0 || $num_aggids == 0 ) {
        printf "cims_agg_update: RateCode or ID array has not been initialized\n";
        return $lines_written;
    }
   
    $count = 0;
    @hkeys = keys(%stat_hash1);
    foreach $hk (@hkeys) {
        $rec_stime = $agg_start_times{$hk};
        $rec_etime = $agg_end_times{$hk};

        ($this_start_date, $this_start_time) = CIMS_lib::cims_GetCSDateTime($rec_stime); 
        ($this_end_date, $this_end_time) = CIMS_lib::cims_GetCSDateTime($rec_etime); 

        printf $my_outhandle "%s,%s,%s,%s,%s,,%s,",$my_aggheader,$this_start_date,$this_end_date,$this_start_time,$this_end_time,$num_aggids;

        $count = 0;
        while ( $count < $num_aggids ) {
            $this_id = $my_aggids[$count];
            ++$count;
            $this_agglist = "id_hash" . $count;
            printf $my_outhandle "%s,\"%s\",",$this_id,$$this_agglist{$hk};            
        }

        printf $my_outhandle "%s,",$num_metrics;            
        $count = 0;
        while ( $count < $num_metrics ) {
            $this_stat = $my_ratecodes[$count];
            ++$count;
            $this_statlist = "stat_hash" . $count;
            if ( $count == $num_metrics ) {
                printf $my_outhandle "%s,%.6f",$this_stat,$$this_statlist{$hk};            
            }
            else {
                printf $my_outhandle "%s,%.6f,",$this_stat,$$this_statlist{$hk};            
            }
        }
        printf $my_outhandle "\n";            
        ++$lines_written;
    }
    return $lines_written;
}






1;
