################################################################################
#
#     Script: CIMS_lib Perl Module
#
#     The information in this  document is subject to  change  without  notice
#     and should not be construed as a commitment by CIMS Lab, Inc.
#     CIMS Lab, Inc.  assumes no responsibility for any errors that
#     may appear in this document.
#
#     The software described in this  document is  furnished to the  purchaser
#     under  a license for use on a single  computer  system and can be copied
#     (with inclusion of CIMS Lab's copyright) only for use in such system,
#     except as  may  otherwise be provided  in writing  from CIMS Lab, Inc.
#
#     All rights reserved.  No part  of this document may be reproduced in any
#     form  or  by any  means,  except as may otherwise be provided in writing
#     from CIMS Lab, Inc.
#
#     Copyright (c) 1979 - 2005 by
#
#     CIMS Lab, Inc., Laurel Lab
#     14440 Cherry Lane Court   Telephone:  (301)-725-2500
#     Suite 201                 Fax      :  (301)-725-7196
#     Laurel, MD 20707
#     USA
#
# The CIMS_lib Perl Module contains modules called by CIMS Process Job scripts.
#
#  CIMS_lib Development History
#
#       Original Author   : George C. Bonifant, CIMS Lab, Inc., Laurel Lab
#       Creation Date     : 10-Feb-2005
#       Last Modification : 14-Jun-2005
#       Version Number    : V1.2
#
#  CIMS_lib Modification Log
#
#       GCB=George C. Bonifant, CIMS Lab, Inc., Laurel Lab
#
#       GCB V1.1 10-Feb-2005
#
#               Created this module.
#
#	GCB V1.2 27-Apr-2005
#
#		Change order of arguments to sort command.
#
#	GCB V1.3 14-Jun-2005
#
#		Set autoflush on STDOUT and STDERR so that
#		output is not buffered.
#
################################################################################
#
package CIMS_lib;

################################################################################
#
%cims_month_hash = (
    "JAN" => "0",
    "FEB" => "1",
    "MAR" => "2",
    "APR" => "3",
    "MAY" => "4",
    "JUN" => "5",
    "JUL" => "6",
    "AUG" => "7",
    "SEP" => "8",
    "OCT" => "9",
    "NOV" => "10",
    "DEC" => "11",
);


################################################################################
#
BEGIN
{
    $log_separator = "*";
    $log_separator x= 80;
    $log_separator .= "\n";
}

################################################################################
################################################################################
#
sub cims_env ()
{
    #
    # get the path to the java executable. /opt/java1.4/jre/bin/ is the
    #  default.
    #
    use FileHandle;

    STDERR->autoflush;
    STDOUT->autoflush;

    $my_jpath = "/opt/java1.4/jre/bin/";
    $my_procdir = "$main::CS_home/sample_processes/";
    if (!open(CIMS_PAR, "$main::CS_home/config/cims.par"))
    {
        die "cims_env: ERROR Can't open CIMS Parameter File\n";
    }
    while ($line = <CIMS_PAR>)
    {
        ($s1, $s2) = split("=",$line);
        if (uc($s1) eq "JAVAPATH")
        {
            $my_jpath = $s2;
            chomp($my_jpath);
            if ( $my_jpath !~ m/(\w+)\/$/ )
            {
                $my_jpath .= "/";
            }
            chomp($my_jpath);
        }
        if (uc($s1) eq "PROCESSESDIR")
        {
            $my_procdir = $s2;
            chomp($my_procdir);
            if ( $my_procdir !~ m/(\w+)\/$/ )
            {
                $my_procdir .= "/";
            }
        }
    }
    close(CIMS_CONF);

    $cims_processdir = $my_procdir;

    $jarfiles .= $main::CS_home;
    $jarfiles .= "/jdbc_drivers/oracle";

    %environ = (
	    "bin" => $main::CS_home . "/bin/",
	    "collectors" => $main::CS_home . "/collectors/",
	    "jlib" => $main::CS_home . "/jlib/",
	    "javapath" => $my_jpath,
	    "lib" => $main::CS_home . "/lib/",
	    "log" => $main::CS_home . "/log/",
	    "processes" => $my_procdir,
	    "scripts" => $main::CS_home . "/scripts/",
        );
    return %environ;
}



################################################################################
################################################################################
#
sub cims_GetDateArg ($)
{
    my ($datearg) = @_;
    if ($datearg eq "PREDAY" or $datearg eq "CURDAY")
    {
        if ($datearg eq "PREDAY")
	{
            my $preday_time = time - 86400;
	    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = 
					localtime($preday_time);
        }
        else   # datearg = CURDAY
        {
	    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = 
					localtime;
        }

        if($mday < 10)
        {
            $mday = "0$mday";
        }
        ++$mon;
        if($mon < 10)
        {
            $mon = "0$mon";
        }
        $year = $year + 1900;
        $value = "$year$mon$mday";
    }
    else
    {
        # Verify date entered on command line is in
        #  the correct format.
        #
        if ( $datearg =~ /(19|20)\d\d[01]\d[0-3]\d/ )
        {
            $value = $datearg;
        }
        else
        {
            $value = "ERROR";
        }
    }
    return $value;
}



################################################################################
################################################################################
#
sub cims_monthhash ($)
{
    ############################################################################
    # This module takes a qualified string, which is a month abreviation, 
    #  i.e. "Jan"
    #  in format used in CIMS CSR records.
    ############################################################################
    #
    my ($month_str) = @_;
    $month_str = uc $month_str ;

    $month_value = $cims_month_hash{$month_str};
    return $month_value;
}



################################################################################
################################################################################
#
sub cims_GetCSDateTime ($)
{
    ############################################################################
    # This module takes an Epoch datetime and returns an array of (date time)
    #  in format used in CIMS CSR records.
    ############################################################################
    #
    my ($epoch_dt) = @_;

    ($seconds, $minutes, $hours, $dom, $month, $year
		, $wday, $yday, $isdst) = localtime($epoch_dt);

    $hours = sprintf "%02d",$hours; 
    $minutes = sprintf "%02d",$minutes; 
    $seconds = sprintf "%02d",$seconds; 
    $my_time = $hours . ":" . $minutes . ":" . $seconds;

    $my_month = $month + 1;
    $my_month = sprintf "%02d",$my_month; 
    $dom = sprintf "%02d",$dom; 
    $my_year = $year + 1900;

    $my_date = $my_year . $my_month . $dom;

    return ($my_date, $my_time);
}



################################################################################
################################################################################
#
sub cims_GetSubDirs ($)
{
    ############################################################################
    # Returns an array of the subdirectories of the
    #  ParentPath. If none are found, an empty list
    #  is returned. If error occurs, return list with
    #  one element, "-ERROR-"
    #
    # The returned list of sub directories have been 
    #  checked to verify, they are directories and 
    #  . and .. have been filtered out of the list.
    ############################################################################
    #
    ($ParentPath) = @_;
    @ret_subdirs = ();
    $found_dir = 0;

    if(!opendir(hDir,$ParentPath)) {
        printf ("%s : ERROR cims_GetSubDirs: Unable to open directory %s\n"
                        ,cims_CurrentDateTime(),$ParentPath);
        push(@ret_subdirs,"-ERROR-");
        return(@ret_subdirs);
    }

    @aDir = readdir(hDir);
    foreach $dirent (@aDir) {
        #
        # Don't look in "." and ".." directories...
        #
        if(($dirent eq ".") || ($dirent eq "..")) {
            next;
        }
        if (( -b "$ParentPath/$dirent" ) || ( -c "$ParentPath/$dirent" )
                 || ( -l "$ParentPath/$dirent" ) || ( -S "$ParentPath/$dirent" )
                 || ( -S "$ParentPath/$dirent" )) {
            next;
        }

        if(-d "$ParentPath/$dirent")  {
            ++$found_dir;
            push(@ret_subdirs,$dirent);
        }
    }

    closedir hDir;
    return(@ret_subdirs);
}




################################################################################
################################################################################
#
sub cims_DoJob ($$)
{
    my ($jobtodo, $processdate) = @_;
    %my_env = cims_env;

    $thisjobdir = $cims_processdir . $jobtodo;
    $jobtodo_script = $thisjobdir . "/Job" . $jobtodo . ".pl";


    printf "$log_separator";
    printf ("%s : INFORMATION cims_DoJob: Excuting job - %s\n"
		,cims_CurrentDateTime(),$jobtodo);
    printf "    Processes Date     : $processdate\n";
    printf "    Processes Directory: $cims_processdir\n";
    printf "    Job Directory      : $thisjobdir\n";
    printf "    Job Script         : $jobtodo_script\n";

    $my_status = system($jobtodo_script,$processdate);

    # return values from system call must be shifted.
    $my_status = $my_status >> 8;

    if($my_status == 0) {
        printf ("%s : INFORMATION cims_DoJob: Completed job - %s\n"
		,cims_CurrentDateTime(),$jobtodo);
        return 0;
    }
    else { 
        if ($my_status > 8) {
            printf ("%s : ERROR cims_DoJob: %s failed\n"
			,cims_CurrentDateTime(),$jobtodo_script);
        }
        else {
            printf ("%s : WARNING cims_DoJob: %s returned warning status\n"
			,cims_CurrentDateTime(),$jobtodo_script);
        }
        return $my_status;
    }
}



################################################################################
################################################################################
#
sub cims_Scan ($$$$$)
{
    print "$log_separator";
    my ($processname, $processdate, $retainDateFlag, $allowEmptyFiles, $allowMissingFiles) = @_;

    $my_status = 0;
    $found_file = 0;
    cims_env;

    printf ("%s : INFORMATION cims_Scan: Begin scan for process - %s\n"
			,cims_CurrentDateTime(),$processname);
    printf "    processdate = $processdate\n";
    printf "    retainDateFlag = $retainDateFlag\n";
    printf "    allowEmptyFiles = $allowEmptyFiles\n";
    printf "    allowMissingFiles = $allowMissingFiles\n";
    $targetDir = "$cims_processdir/$processname";

    ############################################################################
    # Check to see if there's a CurrectCSR.txt file already present.
    #  If there is, rename it to CurrentCSR.last.
    ############################################################################
    #
    if (-e "$targetDir/CurrentCSR.txt") {
        printf ("%s : INFORMATION cims_Scan: Renaming CurrentCSR.txt to CurrentCSR.last\n"
			,cims_CurrentDateTime());
        if (-e "$targetDir/CurrentCSR.last") {
            unlink("$targetDir/CurrentCSR.last");
        }
        rename("$targetDir/CurrentCSR.txt","$targetDir/CurrentCSR.last");
    }

    ############################################################################
    # Open a new CurrentCSR.txt file
    ############################################################################
    #
    if(!open(NEWCSR, ">> $targetDir/CurrentCSR.txt")) {
        printf ("%s : ERROR cims_Scan: Unable to open new CurrentCSR.txt file\n"
			,cims_CurrentDateTime());
        return(16);
    }

    @subdir_list = cims_GetSubDirs($targetDir);
    if ( scalar(@subdir_list) == 0 ) {
        printf ("%s : ERROR cims_Scan: No process sub-directory found\n"
				,cims_CurrentDateTime());
        return(16);
    }
    elsif ( $subdir_list[0] eq "-ERROR-" ) {
        return(16);
    }
    foreach $subdir (@subdir_list) {
        $targetfile = "$targetDir/$subdir/$processdate.txt";
        if (-e "$targetfile") {
            if (-z "$targetfile") {
                if (uc($allowEmptyFiles) eq "TRUE")  {
                    printf ("%s : WARNING cims_Scan: empty CSR file for %s\n"
				,cims_CurrentDateTime(),$subdir);
                    $my_status = 8;
                }
                else {
                    printf ("%s : ERROR cims_Scan: empty CSR file for %s\n"
				,cims_CurrentDateTime(),$subdir);
                    return(16);
                }
            } 
            else {
                ++$found_file;
                printf ("%s : INFORMATION cims_Scan: Found %s.txt for source -  %s\n"
				,cims_CurrentDateTime(),$processdate,$subdir);
                if(!open(SRCIN, "< $targetfile")) {
                    printf ("%s : ERROR cims_Scan: Unable to open %s\n"
				,cims_CurrentDateTime(),$targetfile);
                    return(16);
                }
                while (<SRCIN>) {
                    print(NEWCSR $_);
                }
                close(SRCIN);
            }
        }
        else {
             
            ####################################################################
            # The CSR file does not exist.
            ####################################################################
            #
            if (uc($allowMissingFiles) == "TRUE")  {
                printf ("%s : WARNING cims_Scan: CSR file does not exist for  %s\n"
				,cims_CurrentDateTime(),$subdir);
                $my_status = 8;
            }
            else {
                printf ("%s : ERROR cims_Scan: CSR file does not exist for  %s\n"
				,cims_CurrentDateTime(),$subdir);
                return(16);
            }
        }
    }
    close(NEWCSR);

    printf ("%s : INFORMATION cims_Scan: End scan for process - %s\n"
				,cims_CurrentDateTime(),$processname);
    print "$log_separator";
    return($my_status);
}





################################################################################
################################################################################
#
sub cims_Sort ($$)
{
    my ($infile, $outfile) = @_;

    printf ("%s : INFORMATION cims_Sort: Begin sorting records in %s\n"
				,cims_CurrentDateTime(),$infile);

    if(-e $infile) {
        @sys_array = ("sort","-k","1.24,1.152", "-o", $outfile, $infile );
        $this_status = system(@sys_array);

        # return values from system call must be shifted 8 bits.
        #
        $this_status = $this_status >> 8;
    }
    else {
        printf ("%s : ERROR cims_Sort: %s not found\n"
				,cims_CurrentDateTime(),$infile);
        return(16);
    }
    return($this_status);
}



################################################################################
################################################################################
#
sub cims_CleanUp ($$$$)
{

############################################################################
#
# cims_CleanUp searches the directory indicated by $clean_dir
#  for ordinary text files with a date in their filename. The
#  date is in the format YYYYMMDD. If the date in the filename is
#  chronologically more than $age older than $processdate, the
#  file is removed.
#
# If SubDirs = TRUE, sub-directories of $clean_dir are
#  searched as well.
#
# Arguments:
#
#	$process_date	: Process date in YYYYMMDD format
#	$clean_dir	: directory
#	$Clean_Age	: remove files older than $process_date - $Clean_Age
#       $SubDirs        : If TRUE, clean sub directories
#
############################################################################
#

    print "$log_separator";
    my ($processdate, $clean_dir, $Clean_Age, $SubDirs) = @_;
    printf ("%s : INFORMATION cims_Cleanup: Begin cims_Cleanup\n"
				,cims_CurrentDateTime());
    printf "    LogDate   : $processdate\n";
    printf "    Directory : $clean_dir\n";
    printf "    Clean_Age : $Clean_Age\n";
    printf "    SubDirs   : $SubDirs\n";

    @sub_dirs = ();
    $this_entry = "";
    $files_purged = 0;

    if ( $Clean_Age < 1 ) {
        printf ("%s : INFORMATION cims_Cleanup: cleanup age is less than 1, nothing to do\n"
				,cims_CurrentDateTime());
        return (0);
    }

    if ( -d $clean_dir ) {
    }
    else {
        printf ("%s : ERROR cims_Cleanup: %s is not a directory\n"
				,cims_CurrentDateTime(),$clean_dir);
        return (16);
    }
        
    printf ("%s : INFORMATION cims_Cleanup: Searching directory %s\n"
				,cims_CurrentDateTime(),$clean_dir);
     
    ############################################################################
    # Open the Directory
    ############################################################################
    #
    if(!opendir(hDir,$clean_dir)) {
        printf ("%s : ERROR cims_Cleanup: Unable to open %s directory\n"
				,cims_CurrentDateTime(),$clean_dir);
        return(16);
    }

    ############################################################################
    # Search through the entries in this directory for sub-directories. In each
    #  sub-directory, look for an input source file for the date being 
    #  processed.
    ############################################################################
    #
    @aDir = readdir(hDir);
    foreach $fileent (@aDir) {
        #
        # Don't look in "." and ".." directories...
        #
        if(($fileent eq ".") || ($fileent eq "..")) {
            next;
        }

        $this_entry = "$clean_dir/$fileent";

        if (( -b $this_entry ) || ( -c $this_entry ) || ( -l $this_entry ) || ( -S $this_entry ) || ( -S $this_entry )) {
            next;
        }
        elsif ( -d $this_entry ) {
            push(@sub_dirs,($this_entry));
        }
        elsif ( -T $this_entry ) {
             
            ####################################################################
            # FOUND PLAIN TEXT FILE
            ####################################################################
            #
            if ( $fileent =~ m/(19|20)\d\d[01]\d[0-3]\d/o ) {
                $this_match = $&;
                $this_diff = cims_CompareDates($processdate,$this_match);
                
                if ( $this_diff > $Clean_Age ) {
                    print "cims_CleanUp:     Removing  - $this_entry\n";
                    unlink ($this_entry);
                    ++$files_purged;
                }
            }
        }
        else {
            next;
        }
    }
    closedir hDir;

    $num_subdirs = scalar(@sub_dirs);
    $UC_SubDirs =  uc($SubDirs);
    if ( ( $UC_SubDirs eq "TRUE" ) && ( $num_subdirs > 0 ))  {

        foreach $subdir (@sub_dirs) {
            printf ("%s : INFORMATION cims_Cleanup: Searching sub-directory %s\n"
				,cims_CurrentDateTime(),$subdir);
        
            ####################################################################
            # Open the SubDir 
            ####################################################################
            #
            if(!opendir(hDir,$subdir)) {
                printf ("%s : ERROR cims_Cleanup: Unable to open %s directory\n"
				,cims_CurrentDateTime(),$subdir);
                return(16);
            }

            ####################################################################
            # Search through the entries in this sub-directory. In each
            #  sub-directory, look for any file with YYYYMMDD in it's name.
            ####################################################################
            #
            @aDir = readdir(hDir);
            foreach $fileent (@aDir) {
                #
                # Don't look in "." and ".." directories...
                #
                if(($fileent eq ".") || ($fileent eq "..")) {
                    next;
                }

                $this_entry = "$subdir/$fileent";

                if (( -b $this_entry ) || ( -c $this_entry ) || 
                        ( -l $this_entry ) || ( -S $this_entry ) || 
                        ( -S $this_entry )) {
                    next;
                }
                elsif ( -d $this_entry ) {
                    next;
                }
                elsif ( -T $this_entry ) {
                    #
                    # FOUND PLAIN TEXT FILE
                    if ( $fileent =~ m/(19|20)\d\d[01]\d[0-3]\d/o ) {
                        $this_match = $&;
                        $this_diff = cims_CompareDates($processdate,$this_match);
                
                        if ( $this_diff > $Clean_Age ) {
                            print "cims_CleanUp:     Removing  - $this_entry\n";
                            unlink ($this_entry);
                            ++$files_purged;
                        }
                    }
                }
                else {
                    next;
                }
            }
            closedir hDir;
        }
    }
    printf ("%s : INFORMATION cims_Cleanup: %d Files have been removed\n"
				,cims_CurrentDateTime(),$files_purged);
    printf ("%s : INFORMATION cims_Cleanup: End cims_Cleanup\n"
				,cims_CurrentDateTime());
    print "$log_separator";
}



################################################################################
################################################################################
#
sub cims_CurrentDateTime ()
{
    my ($parent, $child, $rstatus, $hstatus) = @_;
    use Time::localtime;

    $tm = localtime;
    $current_time = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$tm->year+1900,$tm->mon+1,$tm->mday,$tm->hour,$tm->min,$tm->sec);
    return $current_time;
}




################################################################################
################################################################################
#
sub cims_RptStatus ($$$$)
{
    my ($parent, $child, $rstatus, $hstatus) = @_;

    $my_dt = cims_CurrentDateTime();

    if ($rstatus == 0) {
        print "$my_dt : INFORMATION $parent: Returned from $child\n";
        return($hstatus);
    }
    elsif ($rstatus <= 8) {
        print "$my_dt : WARNING $parent: Returned from $child, status = $rstatus\n";
        if ($rstatus > $hstatus) {
            $hstatus = $rstatus;
        }
        return($hstatus);
    }
    else {
        print "$my_dt : ERROR $parent: Returned from $child, status = $rstatus\n";
        return($rstatus);
    }

}



################################################################################
################################################################################
#
sub cims_CompareDates ($$)
{
    ############################################################################
    # Compute the number of days difference in two
    #  dates supplied in YYYYMMDD format.
    #
    # If the second date is chomnologically later than
    #  the first date, the return value is negative.
    ############################################################################
    #
    use Time::Local; 

    my ($date1, $date2) = @_;
    $diff_days = 0;

    $year1 = substr($date1,0,4);
    $year2 = substr($date2,0,4);

    $month1 = substr($date1,4,2);
    $month2 = substr($date2,4,2);

    $day1 = substr($date1,6,2);
    $day2 = substr($date2,6,2);

    $epoch_sec1 = timelocal(1,0,0,$day1,$month1 - 1,$year1 - 1900);
    $epoch_sec2 = timelocal(1,0,0,$day2,$month2 - 1,$year2 - 1900);

    $diff_sec = $epoch_sec1 - $epoch_sec2;
    $diff_days = $diff_sec / 86400;

    return($diff_days);
}



1;
