#!/usr/bin/perl
###########################################################################
#
#     Script: Universal.pl
#
#     The information in this  document is subject to  change  without  notice
#     and should not be construed as a commitment by CIMS Lab, Inc.
#     CIMS Lab, Inc.  assumes no responsibility for any errors that
#     may appear in this document.
#
#     The software described in this  document is  furnished to the  purchaser
#     under  a license for use on a single  computer  system and can be copied
#     (with inclusion of CIMS Lab's copyright) only for use in such system, 
#     except as  may  otherwise be provided  in writing  from CIMS Lab, Inc.
#
#     All rights reserved.  No part  of this document may be reproduced in any
#     form  or  by any  means,  except as may otherwise be provided in writing
#     from CIMS Lab, Inc.
# 
#     Copyright (c) 1979 - 2005 by
# 
#     CIMS Lab, Inc., Laurel Lab
#     14440 Cherry Lane Court	Telephone:  (301)-725-2500
#     Suite 201			Fax      :  (301)-725-7196
#     Laurel, MD 20707
#     USA
#
# The CIMS/UNIX Server Conversion Script.
#
#  (Modified to support converting Apache Access Logs to CSR records)
#
###########################################################################
#  Steps to modify the Universal Conversion Script.
#
#  1. Create a subdirectory in $CIMS_HOME/collectors for this 
#     process definition. i.e. "Apache"
#
#  2. Copy and rename $CIMS_HOME/collectors/Universal/Universal.pl to the newly
#     to the newly created collector sub-directory.
#
#  > cp $CIMS_HOME/collectors/Universal/Universal.pl $CIMS_HOME/collectors/Apache/Apache.pl
#
#  3. Modifiy the new script to extract ids and resources and load them into 
#     the CIMS Aggregation Engine. See notes below marked "CIMS Customer Notes"
#     for location where modifications should be made to the script.
#
#  4. Set the values of $clean_logdirs and $cleanup_age to support 
#     purging the collector sub-directories.
#
###########################################################################
# Usage: Universal.pl <Date> <RetentionFlag> <AllowMissingLogs> <AllowEmptyLogs> <OutputFolder> <LogFolder> [<LogFileName>]
#
#	<Date>		: Date of the CSR files to be loaded into the database.
#		          Date can be PREDAY, CURDAY, or any date in format YYYYMMDD.
#
#	<RetentionFlag> : true/false Cleanup the Collector sub directory?
#
#	<AllowMissingLogs>: true/false If TRUE, return WARNING, else return ERROR.
#
#	<AllowEmptyLogs>: true/false If TRUE, return WARNING, else return ERROR.
#
#	<OutputFolder> 	: Where the CSR file is written.
#
#	<LogFolder>	: Where the raw log file resides.
#
#	[<LogFileName>] : Default is to look for logfile named <Date>.txt . If
#			  logfile is another name, name it here.
#
###########################################################################
#  Conversion Script Modification Log
#
#       GCB=George C. Bonifant, CIMS Lab, Inc., Laurel Lab
#
#       GCB V1.1 28-Mar-2005
#
#               Created this module.
#
################################################################################
#

BEGIN
{
    #
    # Get the directory location where CIMS is installed.
    #
    if (!open(CIMS_CONF, "/etc/cims.conf"))
    {
        die "$0: Can't open CIMS Configuration File\n";
    }
    while ($line = <CIMS_CONF>)
    {
        ($cims_home, $CS_home) = split("=",$line);
        if ($cims_home == "CIMS_HOME")
        {
            last;
        }
    }
    close(CIMS_CONF);

    # Strip of any end of line.
    chomp($CS_home);

    ########################################################################
    # Add the CIMS perl library directory to the lookup
    #  path.
    ########################################################################
    #
    push(@INC, "$CS_home/lib");
    # print "INC = @INC\n\n";
}
  
use Text::ParseWords;
use Time::Local;

use CIMS_lib::CS_libs;
use CIMS_lib::CS_aggtools;

package main;

$hold_status = 0;
$clean_logdirs = "TRUE";
$cleanup_age = 45;

$log_separator = "*";
$log_separator x= 80;
$log_separator .= "\n";

@this_job_arr = split("/",$0);
$this_job = pop(@this_job_arr);

###########################################################################
# Get the CIMS Server Environment Hash
###########################################################################
#
%cims_env = CIMS_lib::cims_env;

###########################################################################
# Check the number of arguments
###########################################################################
#
if( ($#ARGV < 5) || ($#ARGV > 6) ) {
    printf "Usage: %s <Date> <RetentionFlag> <AllowMissingLogs> <AllowEmptyLogs> <OutputFolder> <LogFolder> [<LogFileName>]\n",$this_job;
    exit(16);
}

print "$log_separator";
printf ("%s : INFORMATION %s: Begin Conversion Script%s\n"
                ,CIMS_lib::cims_CurrentDateTime(),$this_job);

###########################################################################
# Initialize program arguments
###########################################################################
#
$process_date = $ARGV[0];
$my_rtflag = $ARGV[1];
$my_allowmissing = $ARGV[2];
$my_allowempty = $ARGV[3];
$my_outfolder = $ARGV[4];
$my_logfolder = $ARGV[5];

printf ("    LogDate           : %s\n",$process_date);
printf ("    RetentionFlag     : %s\n",$my_rtflag);
printf ("    AllowMissingFiles : %s\n",$my_allowmissing);
printf ("    AllowEmptyFiles   : %s\n",$my_allowempty);
printf ("    OutPutFolder      : %s\n",$my_outfolder);
printf ("    LogFolder         : %s\n",$my_logfolder);

###########################################################################
# Check the value of the date argument
###########################################################################
#
$process_date = uc $process_date;
$process_date = CIMS_lib::cims_GetDateArg($process_date);
if ($process_date == "ERROR")
{
    printf ("%s : ERROR %s: invalid <date argument> - %s\n"
                ,CIMS_lib::cims_CurrentDateTime(),$this_job,$process_date);
    exit(16);
}

if( $#ARGV == 6 ) {
    $input_file = $ARGV[6];
    printf ("    LogFile           : %s\n",$my_logfolder);
}
else {
    $input_file = $process_date . ".txt";
}

###########################################################################
# CIMS Customer Note: When creating another collector script
#  from this one, this is where id and resources/metrics are
#  defined to the CIMS Server Aggregation Engine.
#
#  CIMS_aggtools::cims_agg_initheader() is called to define the CSR header
#  for this process.
#
#  CIMS_aggtools::cims_agg_initratecodes() is called to define the
#  ratecodes associated with the resources/metrics.
#
#  CIMS_aggtools::cims_agg_initids() is called to define the id labels
#  that will be associated with the ids.
#
# Initialize the Ratecodes and Identifier Labels.
###########################################################################
#
CIMS_aggtools::cims_agg_clearengine();
CIMS_aggtools::cims_agg_initheader("APACHE");
CIMS_aggtools::cims_agg_initratecodes("APHITS","APBYTES","AP-2","AP-3","AP-4","AP-5");
CIMS_aggtools::cims_agg_initids("Feed","RemoteHost","URL","TOPDIR","NEXTDIR","Referer");

@log_subdirs = CIMS_lib::cims_GetSubDirs($my_logfolder);
if ( scalar(@log_subdirs) == 0) {
    printf ("%s : ERROR %s: no feed sub-directories in %s\n"
                ,CIMS_lib::cims_CurrentDateTime(),$this_job,$my_logfolder);
    exit(16);
}

###########################################################################
# Loop through the Log Folders Sub Directories
###########################################################################
#
foreach $my_feed (@log_subdirs) {

    #######################################################################
    # Open the Log File
    #######################################################################
    #
    $input_fullname = $my_logfolder . "/" . $my_feed . "/" . $input_file;
    if ( ! -f $input_fullname ) {
        if (uc($my_allowmissing) eq "TRUE" ) {
            printf ("%s : WARNING %s: %s does not exist\n"
                    ,CIMS_lib::cims_CurrentDateTime(),$this_job,$input_fullname);
            $hold_status = 8;
            next;
        }
        else {
            printf ("%s : ERROR %s: %s does not exist\n"
                    ,CIMS_lib::cims_CurrentDateTime(),$this_job,$input_fullname);
            exit(16);
        }
    }
    elsif ( -z $input_fullname ) {
        if (uc($my_allowempty) eq "TRUE" ) {
            printf ("%s : WARNING %s: %s is size 0\n"
                    ,CIMS_lib::cims_CurrentDateTime(),$this_job,$input_fullname);
            $hold_status = 8;
            next;
        }
        else {
            printf ("%s : ERROR %s: %s is size 0\n"
                    ,CIMS_lib::cims_CurrentDateTime(),$this_job,$input_fullname);
            exit(16);
        }
    }
    $this_status = open(INFILE, "< $input_fullname");
    if ( $this_status == 0 ) {
        printf ("%s : ERROR %s: can't open %s\n"
                ,CIMS_lib::cims_CurrentDateTime(),$this_job,$input_fullname);
        exit(16);
    }

    printf ("%s : INFORMATION %s: Begin converting to CSR format: %s/%s\n"
                    ,CIMS_lib::cims_CurrentDateTime(),$this_job,$my_feed,$input_file);

    #######################################################################
    # Open the Output File
    #######################################################################
    #
    $outfile_path = $my_outfolder . "/" . $my_feed;
    $outfile_name = $outfile_path . "/" . $process_date . ".txt";

    if ( ! -d $outfile_path ) {
        $this_status = mkdir($outfile_path,0775);
        if ( $this_status == 0 ) {
            printf ("%s : ERROR %s: can't create directory %s\n"
                    ,CIMS_lib::cims_CurrentDateTime(),$this_job,$outfile_path);
            exit(16);
        }
    }

    if ( -f $outfile_name ) {
        unlink ($outfile_name );
    }
    $this_status = open(OUTFILE, "> $outfile_name");
    if ( $this_status == 0 ) {
        printf ("%s : ERROR %s: can't open %s\n"
                ,CIMS_lib::cims_CurrentDateTime(),$this_job,$outfile_name);
        exit(16);
    }

    #######################################################################
    # Process the individual records
    #######################################################################
    #
    $lines_read = 1;
    while ( <INFILE> ) {

        ###################################################################
        # CIMS Customer Note: When implementing another collector script
        #  from this one, this is where the input line of the log file
        #  has been read into $inline. You now split $inline into ids
        #  and resources/metrics to be passed to the CIMS Server Agregation
        #  Engine using CIMS_aggtools::cims_agg_update()
        ###################################################################
        #
        $inline = $_;
        chomp $inline;
        @infields = quotewords(" ",FALSE,($inline));
        $num_fields = scalar(@infields);

        # Default Apache Log contains 8 or 9 fields.
        #   field 9 = Referer (optional)
        #
        if ( ($num_fields != 8 ) && ($num_fields != 9 )) {
            printf ("%s : WARNING %s: invalid log record, line: %s\n"
                    ,CIMS_lib::cims_CurrentDateTime(),$this_job,$lines_read);
            $hold_status = 8;
            ++$lines_read;
            next;
        }
        else {
            $id1 = $my_feed;
            $id2 = $infields[0];

            @tmp_ar = split (" ",$infields[5]);

            $my_index = index($tmp_ar[1],"?");
            if ($my_index == -1) {
                $id3 = $tmp_ar[1];
            }
            else {
                $id3 = substr($tmp_ar[1],0,$my_index);
            }

            @url_info = split ("/",$id3);
            $id4 = $url_info[1];
            $id5 = $url_info[2];

            if ( $num_fields == 9 ) {
                $id6 = $infields[8];
            }
            else {
                $id6 = "";
            }

###########################################################################
# Diagnostic messages
###########################################################################
#
#printf "    id1 = %s\n",$id1;
#printf "    id2 = %s\n",$id2;
#printf "    id3 = %s\n",$id3;
#printf "    id4 = %s\n",$id4;
#printf "    id5 = %s\n",$id5;

            $stat1 = 1;
            if ( $infields[7] eq "-" ) {
                $stat2 = 0;
            }
            else {
                $stat2 = $infields[7];
            }
            $stat3 = 0;
            $stat4 = 0;
            $stat5 = 0;
            $stat6 = 0;

            $tmp_stat = $infields[6];
            if ( ($tmp_stat >= 200) && ($tmp_stat < 300) ) {
                $stat3 = 1;
            }
            elsif ( ($tmp_stat >= 300) && ($tmp_stat < 400) ) {
                $stat4 = 1;
            } 
            elsif ( ($tmp_stat >= 400) && ($tmp_stat < 500) ) {
                $stat5 = 1;
            } 
            elsif ( ($tmp_stat >= 500) && ($tmp_stat < 600) ) {
                $stat6 = 1;
            } 

###########################################################################
# Diagnostic messages
###########################################################################
#
#printf "    stat1 = %s\n",$stat1;
#printf "    stat2 = %s\n",$stat2;
#printf "    stat3 = %s\n",$stat3;
#printf "    stat4 = %s\n",$stat4;
#printf "    stat5 = %s\n",$stat5;
#printf "    stat6 = %s\n",$stat6;

            ############################################################### 
            # Get date and time..
            ###############################################################
            #
            $tmp_string = substr($infields[3],1);
            if ( ! ($tmp_string =~ /^(0|1|2|3)[0-9]\/(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\/(1|2)[0-9][0-9][0-9]:(0|1|2)[0-9]:[0-5][0-9]:[0-5][0-9]/s) ) {
                printf ("%s : WARNING %s: invalid time field: %s , line: %s\n"
                    ,CIMS_lib::cims_CurrentDateTime(),$this_job,$tmp_string,$lines_read);
                ++$lines_read;
                $hold_status = 8;
                next;
            }
            @dtarray = split(":",$tmp_string);
            $my_hour = $dtarray[1];
            $my_min = $dtarray[2];
            $my_sec = $dtarray[3];

            @tmp_dtarray = split("/",$dtarray[0]);
            $my_day = $tmp_dtarray[0];
            $my_month = CIMS_lib::cims_monthhash($tmp_dtarray[1]);
            $my_year = $tmp_dtarray[2];

###########################################################################
# Diagnostic messages
###########################################################################
#
#printf "    my_hour = %s\n",$my_hour;
#printf "    my_min = %s\n",$my_min;
#printf "    my_sec = %s\n",$my_sec;
#printf "    my_day = %s\n",$my_day;
#printf "    my_month = %s\n",$my_month;
#printf "    my_year = %s\n",$my_year;

            $this_dt = timelocal($my_sec,$my_min,$my_hour
                    ,$my_day,$my_month,($my_year - 1900));
            CIMS_aggtools::cims_agg_update($id1,$id2,$id3,$id4,$id5,$id6
                    ,$stat1,$stat2,$stat3,$stat4,$stat5,$stat6
                    ,$this_dt,$this_dt);
            ++$lines_read;
        }
    }
    close(INFILE);
    $lines_written = CIMS_aggtools::cims_agg_genCSR(\*OUTFILE);

    #######################################################################
    # Clear the Aggregation Engine for the next feed logfile.
    #######################################################################
    #
    CIMS_aggtools::cims_agg_clearengine();
    printf ("%s : INFORMATION %s:     %s records read | %s records written\n"
                    ,CIMS_lib::cims_CurrentDateTime(),$this_job,$lines_read,$lines_written);
    close(OUTFILE);
}

if ( uc($clean_logdirs) eq "TRUE") {
    CIMS_lib::cims_CleanUp($process_date,$my_logfolder,$cleanup_age,"true");
}
exit($hold_status);
