#!/usr/bin/perl
################################################################################
#
#     Script: Job<process>.pl
#
#     The information in this  document is subject to  change  without  notice
#     and should not be construed as a commitment by CIMS Lab, Inc.
#     CIMS Lab, Inc.  assumes no responsibility for any errors that
#     may appear in this document.
#
#     The software described in this  document is  furnished to the  purchaser
#     under  a license for use on a single  computer  system and can be copied
#     (with inclusion of CIMS Lab's copyright) only for use in such system,
#     except as  may  otherwise be provided  in writing  from CIMS Lab, Inc.
#
#     All rights reserved.  No part  of this document may be reproduced in any
#     form  or  by any  means,  except as may otherwise be provided in writing
#     from CIMS Lab, Inc.
#
#     Copyright (c) 1979 - 2005 by
#
#     CIMS Lab, Inc., Laurel Lab
#     14440 Cherry Lane Court           Telephone:  (301)-725-2500
#     Suite 201                 Fax      :  (301)-725-7196
#     Laurel, MD 20707
#     USA
#
# The CIMS/UNIX Server process job script performs the following tasks. 
#
#     1. If an external feed collection script exists for this process,
#        that script is executed. The collection script is located in
#        $CIMS_HOME/collectors/$this_job. Note: not all processes use
#        a collection script.
#
#     2. CIMS_scan is called to gather the <date>.txt files contained in 
#        the process sub-directories to created CurrentCSR.txt in the process 
#        directory. 
#
#     3. Call procCIMS.pl to run CIMSAcct, CIMSsort, CIMSBill, and CIMSLoad 
#        jobs for this process.
#
#     4. CIMS_cleanup is called to purge the process sub-directories.
#
# Usage: Job<process>.pl <date>
#
#       <date>  : Date of the CSR files to be loaded into the database.
#                 Date can be PREDAY, CURDAY, or any date in format YYYYMMDD.
#
# The following variables must be initialized below...
#
#       this_job	 : Name of the Process. i.e. "UnixOS"
#       job_xml		 : Directives for running CIMSAcct, CIMSBill, and 
#			   CIMSLoad.
#	retainDateFlag	 : Keep a local copy of raw accounting logs
#       allowEmptyFiles  : IF TRUE, return Warning level status if Input CSR 
#			   file is empty, else return Error level status. 
#       allowMissingFiles: If TRUE, return Warning level status if Input CSR 
#			   file is missing, else return Error level status. 
#       Clean_Age	 : Number of <date>.txt files to keep on-line.
#                          0 indicates no files are to be purged.
#	SubDirs		 : If TRUE, clean/(purge) sub-directories of the
#			   processes directory.
#
################################################################################
#  Job<process>.pl Modification Log
#
#       GCB=George C. Bonifant, CIMS Lab, Inc., Laurel Lab
#
#       GCB V1.1 28-Mar-2005
#
#               Created this module.
#
################################################################################
#

$this_job = "UnixOS";
$job_xml = "ProcCntl.xml";

$retainDateFlag = "FALSE";
$allowEmptyFiles = "TRUE";
$allowMissingFiles = "TRUE";
$Clean_Age = 100;
$SubDirs = "TRUE";

################################################################################
# Get the directory location where CIMS is installed.
#  and add the CIMS lib directory to the search path.
################################################################################
#
BEGIN {
if (!open(CIMS_CONF, "/etc/cims.conf"))
{
    die "$0: Can't open CIMS Configuration File\n";
}
while ($line = <CIMS_CONF>)
{
    ($cims_home, $CS_home) = split("=",$line);
    if ($cims_home == "CIMS_HOME")
    {
        last;
    }
}
close(CIMS_CONF);

# Strip of any end of line.
chop($CS_home);
push(@INC, "$CS_home/lib");
}

################################################################################
# Define the CS_libs modules
################################################################################
#
use CIMS_lib::CS_libs;

$this_script = "Job" . $this_job . ".pl";
$hold_status = 0;

$log_separator = "*";
$log_separator x= 80;
$log_separator .= "\n";

################################################################################
# Check to see the required number of arguments are present.
################################################################################
#
if($#ARGV lt 0)
{
    print "Usage: $this_script <date>\n";
    exit(16);
}

################################################################################
# Get and verify the date entered on the command line.
################################################################################
#
$process_date = $ARGV[0];
$process_date = uc $process_date;
$process_date = CIMS_lib::cims_GetDateArg($process_date);
if ($process_date == "ERROR")
{
    print "$this_script:    Invalid date entered on comand line\n";
    exit(16);
}

################################################################################
# Display CIMS configuration
################################################################################
#
print "$log_separator";
(@tempwords) = split "/", $0;
$scriptname = pop @tempwords;

printf ("%s : INFORMATION %s: Begin Process - %s\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script,$this_job);
printf ("%s : INFORMATION %s:    Process Date = %s\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script,$process_date);;

my %cims_env = CIMS_lib::cims_env;
print "    CIMS environment: Collectors = $cims_env{collectors}\n";
print "    CIMS environment: Java Path  = $cims_env{javapath}\n";
print "    CIMS environment: Lib        = $cims_env{lib}\n";
print "    CIMS environment: Processes  = $cims_env{processes}\n";
print "    CIMS environment: Scripts    = $cims_env{scripts}\n";

################################################################################
# CALLING Conversion script
################################################################################
#
printf ("%s : INFORMATION %s: Call Conversion script for %s\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script,$this_job);

if ( -d "$cims_env{collectors}$this_job" ) {
    if ( -x "$cims_env{collectors}$this_job/$this_job.pl" ) {
        $this_collector_job = "$cims_env{collectors}$this_job/$this_job.pl";
        $process_folder = $cims_env{processes} . $this_job;
        $collector_folder = $cims_env{collectors} . $this_job;
        @job_array = ("$this_collector_job","$process_date","$retainDateFlag"
                        ,"$allowMissingFiles","$allowEmptyFiles"
                        ,"$process_folder","$collector_folder");
        $this_status = system(@job_array);
        $this_status = $this_status >> 8;  # return from system() must be shifted
        $hold_status = CIMS_lib::cims_RptStatus($this_script,"$this_job.pl"
			,$this_status,$hold_status);
        if ($hold_status > 8) {
            exit($this_status);
        }
    }
    else {
        printf ("%s : INFORMATION %s: No Conversion script present\n"
			,CIMS_lib::cims_CurrentDateTime(),$this_script);
    }
}
else {
    printf ("%s : INFORMATION %s: No collector directory present\n"
			,CIMS_lib::cims_CurrentDateTime(),$this_script);
}


################################################################################
# CALLING CIMSScan
################################################################################
#
printf ("%s : INFORMATION %s: Call cims_Scan\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script);

$this_status = CIMS_lib::cims_Scan($this_job,$process_date,$retainDateFlag
		,$allowEmptyFiles,$allowMissingFiles);

$hold_status = CIMS_lib::cims_RptStatus($this_script,"cims_Scan",$this_status
		,$hold_status);
if ($hold_status > 8) {
    exit($this_status);
}

################################################################################
# CALLING ProcCIMS
################################################################################
#
printf ("%s : INFORMATION %s: Call procCIMS.pl\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script);
@job_array = ("$cims_env{scripts}procCIMS.pl", "$this_job", "$job_xml");
$this_status = system(@job_array);

$this_status = $this_status >> 8;     # return from system() must be shifted

$hold_status = CIMS_lib::cims_RptStatus($this_script,"procCIMS.pl",$this_status
		,$hold_status);
if ($hold_status > 8) {
    exit($this_status);
}

################################################################################
# CALLING CIMSCleanup
################################################################################
#
printf ("%s : INFORMATION %s: Call cims_Cleanup\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script);

$in_folder = "$cims_env{processes}$this_job";
CIMS_lib::cims_CleanUp($process_date,$in_folder,$Clean_Age,$SubDirs);

printf ("%s : INFORMATION %s: End Process - %s\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script,$this_job);
print "$log_separator";
exit($hold_status);
