#!/usr/bin/perl
#
#     Script: nightly.pl
#
#     The information in this  document is subject to  change  without  notice
#     and should not be construed as a commitment by CIMS Lab, Inc.
#     CIMS Lab, Inc.  assumes no responsibility for any errors that
#     may appear in this document.
#
#     The software described in this  document is  furnished to the  purchaser
#     under  a license for use on a single  computer  system and can be copied
#     (with inclusion of CIMS Lab's copyright) only for use in such system, 
#     except as  may  otherwise be provided  in writing  from CIMS Lab, Inc.
#
#     All rights reserved.  No part  of this document may be reproduced in any
#     form  or  by any  means,  except as may otherwise be provided in writing
#     from CIMS Lab, Inc.
# 
#     Copyright (c) 1979 - 2005 by
# 
#     CIMS Lab, Inc., Laurel Lab
#     14440 Cherry Lane Court	Telephone:  (301)-725-2500
#     Suite 201			Fax      :  (301)-725-7196
#     Laurel, MD 20707
#     USA
###########################################################################
#
# The CIMS Server for Unix Nightly Perl Script is provided by CIMS Lab, Inc.
#  as a template. You must configure this script to execute the process job
#  steps required for your environemnt.
#
# Usage: nightly.pl <date>
#
#	<date>	: Date of the CSR files to be loaded into the database.
#		  Date can be PREDAY, CURDAY, or any date in format YYYYMMDD.
#
#
# Configurables:
#
#		stopOnProcessFailure : If set to TRUE, this job will
#			stop if a process step returns a failure status.
#			If set to FALSE, this job will continue to the
#			next process step.
#
###########################################################################
#  nightly.pl Modification Log
#
#       GCB=George C. Bonifant, CIMS Lab, Inc., Laurel Lab
#
#       GCB V1.1 10-Feb-2005
#
#               New script.
#
###########################################################################
#

BEGIN
{
    #######################################################################
    # Get the directory location where CIMS is installed.
    #######################################################################
    #
    if (!open(CIMS_CONF, "/etc/cims.conf"))
    {
        die "$0: Can't open CIMS Configuration File\n";
    }
    while ($line = <CIMS_CONF>)
    {
        ($cims_home, $CS_home) = split("=",$line);
        if ($cims_home == "CIMS_HOME")
        {
            last;
        }
    }
    close(CIMS_CONF);

    #######################################################################
    # Strip of any end of line.
    #######################################################################
    #
    chop($CS_home);

    #######################################################################
    # Add the CIMS perl library directory to the lookup
    #  path.
    #######################################################################
    #
    push(@INC, "$CS_home/lib");
}

use CIMS_lib::CS_libs;

package main;

$stopOnProcessFailure = "FALSE";
#@job_list = ("UnixOS", "UnixORA");
#@job_list = ("UnixOS");
@job_list = ("UnixOS", "UnixORA", "UnixORAstorage", "UnixDB2", "UnixDB2storage", "UnixFS", "Apache", "FTPD");

$hold_status = 0;
$log_separator = "*";
$log_separator x= 80;
$log_separator .= "\n";
CIMS_lib::cims_env;

#
# Don't suspend the job if run in background
#
local $SIG{TTOU} = "IGNORE";
print "\n";

###########################################################################
# Check to see the required number of arguments are present.
###########################################################################
#
if($#ARGV lt 0)
{
    print "Usage: nightly.pl <date>\n";
    exit;
}

print "$log_separator";

printf ("%s : INFORMATION Begin - $0\n",CIMS_lib::cims_CurrentDateTime());

$arg_count = 0;
foreach $argument (@ARGV)
{
    print "    ARGV $arg_count: $ARGV[$arg_count]\n";
    ++$arg_count;
}

###########################################################################
# Get and verify the date entered on the command line.
###########################################################################
#
$process_date = $ARGV[0];
$process_date = uc $process_date;
$process_date = CIMS_lib::cims_GetDateArg($process_date);
if ($process_date == "ERROR")
{
    printf ("%s : ERROR Invalid date entered on comand line\n",CIMS_lib::cims_CurrentDateTime());
    exit(16);
}
print "    Process Date: $process_date\n";


###########################################################################
###########################################################################
###########################################################################
# Call DoJob for each Process.
###########################################################################
#

foreach $job (@job_list) {
    print "$log_separator";
    $this_status = CIMS_lib::cims_DoJob $job, $process_date;
    $hold_status = CIMS_lib::cims_RptStatus($0, "cims_DoJob", $this_status, $hold_status);
    if ( ($hold_status > 8) && (uc($stopOnProcessFailure) eq "TRUE") ) {
        exit($hold_status);
    }
}

print "$log_separator";
printf ("%s : INFORMATION $0: Done\n",CIMS_lib::cims_CurrentDateTime());

exit ($hold_status);
