#!/usr/bin/perl
#
#     Script: procCIMS.pl
#
#     The information in this  document is subject to  change  without  notice
#     and should not be construed as a commitment by CIMS Lab, Inc.
#     CIMS Lab, Inc.  assumes no responsibility for any errors that
#     may appear in this document.
#
#     The software described in this  document is  furnished to the  purchaser
#     under  a license for use on a single  computer  system and can be copied
#     (with inclusion of CIMS Lab's copyright) only for use in such system, 
#     except as  may  otherwise be provided  in writing  from CIMS Lab, Inc.
#
#     All rights reserved.  No part  of this document may be reproduced in any
#     form  or  by any  means,  except as may otherwise be provided in writing
#     from CIMS Lab, Inc.
# 
#     Copyright (c) 1979 - 2005 by
# 
#     CIMS Lab, Inc., Laurel Lab
#     14440 Cherry Lane Court		Telephone:  (301)-725-2500
#     Suite 201			Fax      :  (301)-725-7196
#     Laurel, MD 20707
#     USA
#
# The CIMS Server for Unix Process CIMS Script is provided by CIMS Lab, Inc. 
#  as a template. You must configure this script to execute the process jobs 
#  required for your environemnt.
#
# Usage: procCIMS.pl <ProcessName> <XML_JobFile>
#
#	<ProcessName> 	: The name of the process for which procCIMS.pl will 
#                 	  execute CIMSAcct, CIMSSort, CIMSBill, and CIMSLoad.   
#
#	<XML_JobFile>	: The name of the XML Job File which defines
#			  parameters to CIMSAcct, CIMSSort, CIMSBill, and 
#			  CIMSLoad.
#
#  This script will look for a directory within the CIMS Server for Unix
#  processes folder for a sub-directory named <ProcessName>. The XML Job
#  File must be present in this directory. $CIMS_HOME/processes/<ProcessName>
#
#

BEGIN
{
    #
    # Get the directory location where CIMS is installed.
    #
    if (!open(CIMS_CONF, "/etc/cims.conf"))
    {
        die "$0: Can't open CIMS Configuration File\n";
    }
    while ($line = <CIMS_CONF>)
    {
        ($cims_home, $CS_home) = split("=",$line);
        if ($cims_home == "CIMS_HOME")
        {
            last;
        }
    }
    close(CIMS_CONF);

    # Strip of any end of line.
    chop($CS_home);

    #
    # Add the CIMS perl library directory to the lookup
    #  path.
    #
    push(@INC, "$CS_home/lib");
    # print "INC = @INC\n\n";
}

use CIMS_lib::CS_libs;

package main;

$log_separator = "*";
$log_separator x= 80;
$log_separator .= "\n";
print "$log_separator";

$this_script = "ProcCIMS.pl";
$exitOnAcctWarning = "TRUE";

$hold_status = 0;

my %cims_env = CIMS_lib::cims_env;
$ENV{CLASSPATH} = $cims_env{classpath};

#
# Don't suspend the job if run in background
#
local $SIG{TTOU} = "IGNORE";

################################################################################
# Check to see the required number of arguments are present.
################################################################################
#
if($#ARGV ne 1)
{
    printf ("%s : ERROR Usage: procCIMS.pl <process-folder> <job-xml-file>\n"
                ,CIMS_lib::cims_CurrentDateTime());
    exit(16);
}

################################################################################
# Get and verify the Job process entered on the command line.
################################################################################
#
$this_job = $ARGV[0];
if ( -d "$cims_env{processes}$this_job" ) {
    printf ("%s : INFORMATION %s: Begin procCIMS for process - %s\n"
                  ,CIMS_lib::cims_CurrentDateTime(),$this_script,$this_job);
}
else {
    printf ("%s : ERROR %s: %s%s directory not found\n"
                ,CIMS_lib::cims_CurrentDateTime(),$this_script,$cims_env{processes}
		,$this_job);
    exit(16);
}

$job_xml_file = $ARGV[1];

################################################################################
# CALLING CIMSAcct
################################################################################
#
print "$log_separator";
printf ("%s : INFORMATION %s:    Call CIMSAcct\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script);

@test_array = ("$cims_env{javapath}java", "-jar", "$cims_env{jlib}CIMSAcct.jar", "$cims_env{processes}$this_job", $job_xml_file);
#print "@test_array\n";
$this_status = system(@test_array);

# return values from system call must be shifted 8 bits.
#
$this_status = $this_status >> 8;

$hold_status = CIMS_lib::cims_RptStatus($this_script, "CIMSAcct", $this_status, $hold_status);
if ($hold_status > 8) {
    exit($this_status);
}
elsif ($this_status != 0) {
    if (uc($exitOnAcctWarning) eq "TRUE") {
        exit($this_status);
    }
}

################################################################################
# CALLING CIMSSort
################################################################################
#
print "$log_separator";
printf ("%s : INFORMATION %s:    Call cims_Sort\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script);

$sort_infile = $cims_env{processes} . "$this_job/AcctCSR.txt";
$sort_outfile = $cims_env{processes} . "$this_job/sortedAcctCSR.txt";

$this_status = CIMS_lib::cims_Sort($sort_infile, $sort_outfile);

$hold_status = CIMS_lib::cims_RptStatus($this_script, "cims_Sort", $this_status, $hold_status);
if ($hold_status > 8) {
    exit($this_status);
}

#
# Remove the original infile and replace it with the new 
#  sorted file.
#
unlink ($sort_infile);
rename ($sort_outfile, $sort_infile);

################################################################################
# CALLING CIMSBill
################################################################################
#
print "$log_separator";
printf ("%s : INFORMATION %s:    Call CIMSBill\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script);

@test_array = ("$cims_env{javapath}java", "-jar", "$cims_env{jlib}CIMSBill.jar", "$cims_env{processes}$this_job", $job_xml_file);
#print "@test_array\n";
$this_status = system(@test_array);

# return values from system call must be shifted 8 bits.
#
$this_status = $this_status >> 8;

$hold_status = CIMS_lib::cims_RptStatus($this_script, "CIMSBill", $this_status, $hold_status);
if ($hold_status > 8) {
    exit($this_status);
}

################################################################################
# CALLING CIMSLoad
################################################################################
#
print "$log_separator";
printf ("%s : INFORMATION %s:    Call CIMSLoad\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script);

@test_array = ("$cims_env{javapath}java", "-jar", "$cims_env{jlib}CIMSLoad.jar", "$cims_env{processes}/$this_job", $job_xml_file);
#print "@test_array\n";
$this_status = system(@test_array);

# return values from system call must be shifted 8 bits.
#
$this_status = $this_status >> 8;

$hold_status = CIMS_lib::cims_RptStatus($this_script, "CIMSLoad", $this_status, $hold_status);
if ($hold_status > 8) {
    exit($this_status);
}

################################################################################
# procCIMS Completed
################################################################################

printf ("%s : INFORMATION %s:    End procCIMS for process - %s\n"
		,CIMS_lib::cims_CurrentDateTime(),$this_script,$this_job);
exit($hold_status);


